#include "globals.h"
#include "ScaffoldAssembler.h"
#include "Reader.h"

#include <cstdio>
#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

static Alignment aux_alignments[1000000];

static bool Compare(const Contig &x, const Contig &y)
{
    return x.Size() > y.Size();
}

void ScaffoldAssembler::Assemble(vector<Contig> &contigs)
{
    kmerLength = option.maxk;
    scaffold_graph.SetMinPairs(option.min_pairs);
    FastAReader contig_reader(option.contigfile);

    Contig contig;
    string comment;
    while (contig_reader.Read(contig, comment))
    {
        int index = comment.size()-1;
        while (isdigit(comment[index]))
            --index;
        contig.sum_coverage = atoi(comment.c_str() + index + 1);
        contig.Encode();
        contigs.push_back(contig);
    }

    cout << "contigs " << contigs.size() << endl;

    sort(contigs.begin(), contigs.end(), Compare);
    //reverse(contigs.begin(), contigs.end());

    scaffold_graph.Initialize(contigs);

    cout << "scaffold graph initialize" << endl;
    //aligner.Initialize(scaffold_graph.GetContigNodes());
    //aligner.Initialize(contigs);
    cout << "aligner initialize" << endl;

    Sequence seq1, seq2;

    FILE *falign = fopen(option.align_file.c_str(), "rb");

    //cout << falign << endl;
    vector<Alignment> alignments1, alignments2;
    while (true)
    {
        int size;
        ///cout << fread(&size, sizeof(int), 1, falign) << endl;
        if (fread(&size, sizeof(int), 1, falign) == 0)
            break;

        alignments1.resize(size);
        fread(aux_alignments, sizeof(Alignment), size, falign);
        copy(aux_alignments, aux_alignments + size, alignments1.begin());

        if (fread(&size, sizeof(int), 1, falign) == 0)
            break;
        alignments2.resize(size);
        fread(aux_alignments, sizeof(Alignment), size, falign);
        copy(aux_alignments, aux_alignments + size, alignments2.begin());

        //cout << alignments1.size() << " " << alignments2.size() << endl;

        scaffold_graph.AddPair(alignments1, alignments2);
    }

    cout << "aligned" << endl;

//    while (true)
//    {
//        vector<Read> reads = GetShortReads();
//        if (reads.size() == 0)
//            break;
//
//        for (int64 i = 0; i < (int64)reads.size(); i += 2)
//        {
//            if (!reads[i].IsActive() || !reads[i+1].IsActive())
//                continue;
//
//            vector<Alignment> alignments1;
//            vector<Alignment> alignments2;
//            seq1 = reads[i];
//            seq2 = reads[i+1];
//
//            aligner.AlignSequence(&seq1, alignments1);
//            aligner.AlignSequence(&seq2, alignments2);
//
//            scaffold_graph.AddPair(alignments1, alignments2);
//        }
//    }
    scaffold_graph.ComputeDistance();

    if (option.readfile2 != "")
    {
        FastAReader reader(option.readfile2);

        string comment1;
        string comment2;
        while (reader.Read(seq1, comment1) && reader.Read(seq2, comment2))
        {
            if (!seq1.IsChar() || !seq2.IsChar())
                continue;

            seq1.Encode();
            seq2.Encode();

            vector<Alignment> alignments1;
            vector<Alignment> alignments2;

            aligner.AlignSequence(&seq1, alignments1);
            aligner.AlignSequence(&seq2, alignments2);

            scaffold_graph.AddPair(alignments1, alignments2, 1);
        }

        cout << "read2 aligned" << endl;
        scaffold_graph.ComputeDistance(1);
        cout << "ok" << endl;
    }

    cout << "aligned" << endl;
    scaffold_graph.FindUniquePaths();
    scaffold_graph.Scaffold(contigs);
}
