#include "globals.h"
#include "BlatRecord.h"
#include "Utils.h"
#include "Sequence.h"
#include "Reader.h"

#include <iostream>
#include <algorithm>
#include <cstdio>
#include <map>
#include <string>

using namespace std;

char buf[MaxLine];
char line[MaxLine];
map<string, int> table;

int threshold = 10;

bool PercentLessThan(int length, int from, int to, double rate)
{
    if (to - from < rate * length)
        return true;
    else 
        return false;
}

bool IsWhole(int length, int from, int to)
{
    if (from <= threshold && to >= length - threshold)
        return true;
    else
        return false;
            
}

bool IsEnd(int length, int from, int to)
{
    if (from <= threshold || to >= length - threshold)
        return true;
    else
        return false;
}

string OverlapType(BlatRecord &record)
{
    if (IsWhole(record.query_length, record.query_from, record.query_to))
    {
        if (IsWhole(record.ref_length, record.ref_from, record.ref_to))
            return "identical";
        else
            return "be contained";
    }
    else if (IsWhole(record.ref_length, record.ref_from, record.ref_to))
    {
        return "contain";
    }
    else if (IsEnd(record.query_length, record.query_from, record.query_to)
            && IsEnd(record.ref_length, record.ref_from, record.ref_to))
        return "overlap";
    else
        return "partial";
}

int main(int argc, char *argv[])
{
    if (argc < 2)
    {
        fprintf(stderr, "analyzeBgiMetaPair blat-file\n");
        exit(1);
    }

    FILE *fblat = fopen(argv[1], "rb");
    fgets(line, MaxLine, fblat);
    fgets(line, MaxLine, fblat);
    fgets(line, MaxLine, fblat);
    fgets(line, MaxLine, fblat);
    fgets(line, MaxLine, fblat);

    string last_query;
    int num_edge = 0;
    while (fgets(line, MaxLine, fblat) != NULL)
    {
        BlatRecord record;
        record.Parse(line);

        if (record.blocks.size() == 1 && record.ref_to - record.ref_from > 30)
        {
            ++num_edge;
//            printf("%lld %lld %lld %lld %lld %lld %s\n",
//                    record.query_length, record.query_from, record.query_to,
//                    record.ref_length, record.ref_from, record.ref_to,
//                    OverlapType(record).c_str());
            table[OverlapType(record)]++;

            if (OverlapType(record) == "overlap")
            {
                if (PercentLessThan(record.query_length, record.query_from, record.query_to, 0.5)
                        && PercentLessThan(record.ref_length, record.ref_from, record.ref_to, 0.5))
                {
                    printf("%lld %lld %lld %lld %lld %lld %s\n",
                            record.query_length, record.query_from, record.query_to,
                            record.ref_length, record.ref_from, record.ref_to,
                            OverlapType(record).c_str());
                }
            }
        }
    }

    cout << num_edge << endl;
    map<string, int>::iterator iter = table.begin();
    while (iter != table.end())
    {
        cout << iter->first << " " << iter->second << endl;
        ++iter;
    }

    return 0;
}

