#include "globals.h"
#include "Log.h"
#include "Sequence.h"
#include "Utils.h"
#include "HashGraph.h"
#include "KmerNode.h"

#include <iostream>
#include <algorithm>
#include <cstdio>
#include <cctype>
#include <cstring>
#include <set>
#include <map>

using namespace std;

struct Interval
{
    int from, to;
};

struct RNA
{
    string name;
    int size;
    vector<Interval> extrons;
    Sequence seq;
};

char buf[MaxLine];
char line[MaxLine];
char name[MaxLine];
char label[MaxLine];
char value[MaxLine];
RNA transcriptome[100000];
int num[1000] = {0};
int len[] = { 0, 25, 50, 75, 100, 200, 300, 500, 1000, 2000, 5000, 10000000};

bool ReadLine(FILE *fp)
{
    label[0] = '\0';
    if (fgets(line, MaxLine, fp) == NULL)
        return false;

    sscanf(line, "%s %s", label, value);

    return true;
}

bool ReplaceDot(char *s)
{
    int count = 0;
    for (int i = 0; s[i]; ++i)
    {
        if (s[i] == '.')
        {
            s[i] = ' ';
            ++count;
        }
    }

    return count == 2;
}

void ToUpper(char *s)
{
    for (int i = 0; s[i]; ++i)
    {
        s[i] = toupper(s[i]);
    }
}

bool ReadRNA(FILE *fp, RNA &rna)
{
    while (ReadLine(fp))
    {
        if (strcmp(label, "LOCUS") == 0)
            break;
    }

    if (strcmp(label, "LOCUS") == 0)
        rna.name = value;
    else
        return false;

    rna.extrons.resize(0);

    while (ReadLine(fp))
    {
        if (strcmp(label, "source") == 0)
        {
            ReplaceDot(value);
            sscanf(value, "%s %d", buf, &rna.size);
        }
        else if (strcmp(label, "exon") == 0)
        {
            if (!isdigit(value[0]))
                continue;

            if (!ReplaceDot(value))
                continue;
            Interval interval;
            sscanf(value, "%d %d", &interval.from, &interval.to);
            //cout << value << endl;
            rna.extrons.push_back(interval);
        }
        else if (strcmp(label, "ORIGIN") == 0)
        {
            string seq;
            while (fscanf(fp, "%s", buf))
            {
                ToUpper(buf);
                if (isdigit(buf[0]))
                    continue;
                else
                {
                    seq += buf;
                    if ((int)seq.size() == rna.size)
                        break;
                }
            }

            rna.seq = seq;
            //rna.seq.SetContent(seq.c_str(), seq.size());

            break;
        }
    }

    return true;
}

int main(int argc, char *argv[])
{
    cout << sizeof(KmerNode) << endl;
    AddParameter("kmer", &kmerLength, INTEGER);

    ProcessParameters(argc, argv);

    int total = 0;
    RNA rna;

    while (ReadRNA(stdin, rna))
    {
        transcriptome[total++] = rna;
    }

    printf("total rna %d\n", total);
    
    int totalExtron = 0;
    for (int i = 0; i < total; ++i)
    {
        totalExtron += transcriptome[i].extrons.size();
    }

    printf("total extrons %d\n", totalExtron);

    map<Sequence, int> exonPool;
    for (int i = 0; i < total; ++i)
    {
        for (unsigned j = 0; j < transcriptome[i].extrons.size(); ++j)
        {
            Interval intv = transcriptome[i].extrons[j];
            int from = intv.from - 1;
            int to = intv.to - 1;

            Sequence seq;
            transcriptome[i].seq.GetSubSequence(seq, from, to - from + 1);

            if (!seq.IsChar())
                continue;

            Sequence revComp = seq;
            revComp.ReverseComplement();
            if (revComp < seq)
                seq = revComp;

            exonPool[seq]++;
        }
    }

    printf("total different extrons %d\n", (int)exonPool.size());

    map<Sequence, int>::iterator iter = exonPool.begin();
    while (iter != exonPool.end())
    {
        num[iter->second]++;
        ++iter;
    }

    for (int i = 1; i <= 20; ++i)
        cout << i << " " << num[i] << endl;
    cout << endl;

    fill_n(num, 1000, 0);
    iter = exonPool.begin();
    while (iter != exonPool.end())
    {
        if (iter->second > 1)
        {
            int length = iter->first.Size();
            int k = 0;
            while (length > len[k+1])
                ++k;
            ++num[k];
        }

        ++iter;
    }

    for (int i = 0; len[i] < 1000000; ++i)
        cout << len[i] << "-" << len[i+1] << " " << num[i] << endl;

    map<Kmer, int> kmerPool;
    iter = exonPool.begin();
    while (iter != exonPool.end())
    {
        if (iter->second > 1)
        {
            Sequence seq = iter->first;
            Kmer kmer;

            for (int i = 0; i+1 < kmerLength; ++i)
                kmer.AddRight(seq[i]);
            for (int i = kmerLength - 1; i < seq.Size(); ++i)
            {
                kmer.AddRight(seq[i]);

                kmerPool[kmer]++;
            }
        }

        ++iter;
    }

    int count = 0;
    map<Kmer, int>::iterator iter2 = kmerPool.begin();
    while (iter2 != kmerPool.end())
    {
        if (iter2->second > 1)
            ++count;
        iter2++;
    }
    cout << count << endl;

    return 0;
}
