#include "globals.h"
#include "Sequence.h"
#include "Log.h"
#include "Reader.h"
#include "Writer.h"
#include "Utils.h"

#include <iostream>
#include <algorithm>
#include <cstdio>
#include <vector>
#include <cstring>
#include <set>
#include <string>
#include <map>

using namespace std;

const int MaxSeqLength = 10000000;

int reverse_table[256] = {0};
char buf[MaxLine];
char line[MaxLine];
char readName[MaxLine];
char contigName[MaxLine];
char comment[MaxLine];
char strand[MaxLine];
vector<string> names;
vector<Sequence> contigs;
map<string, int> dict;
vector<vector<int> > matchs;
vector<int> aligned_bases;
long long errors[1000000] = {0};
long long partial[1000000] = {0};
long long cover_count[10000] = {0};
long long cover_length[10000] = {0};
bool is_multi = false;

int main(int argc, char *argv[])
{
    AddParameter("multi", &is_multi, SIMPLE);
    ProcessParameters(argc, argv);

    reverse_table['A'] = 'T';
    reverse_table['C'] = 'G';
    reverse_table['G'] = 'C';
    reverse_table['T'] = 'A';
    reverse_table['N'] = 'N';

    if (argc < 3
        || strcmp(argv[1], "--help") == 0
        || strcmp(argv[1], "-h") == 0)
    {
        fprintf(stderr, "usage: calcCover contig-file read-file blat-file\n");
        fprintf(stderr, "       [--minContig l]\n");
        exit(1);
    }

    FastAReader contigs_reader(argv[1]);
    Sequence seq;
    string comment;
    while (contigs_reader.Read(seq, comment))
    {
        sscanf(comment.c_str(), "%s", contigName);
        dict[contigName] = contigs.size();
        contigs.push_back(seq);
        names.push_back(contigName);
    }

    matchs.resize(contigs.size());
    for (unsigned i = 0; i < contigs.size(); ++i)
        matchs[i].resize(contigs[i].Size(), 0);

    FastAReader reader(argv[2]);
    FILE *fp = fopen(argv[3], "rb");

    fgets(line, MaxLine, fp);
    fgets(line, MaxLine, fp);
    fgets(line, MaxLine, fp);
    fgets(line, MaxLine, fp);
    fgets(line, MaxLine, fp);

    Sequence current_read;
    string current_read_name;
    long long total_aligned_reads = 0;
    long long total_aligned_reads_length = 0;

    while (fgets(line, MaxLine, fp) != NULL)
    {
        int match_count = 0;
        int read_from, read_to;
        int read_length = 0;
        int contig_from, contig_to;
        int contig_length;
        sscanf(line, "%d %s %s %s %s %s %s %s %s %s %d %d %d %s %d %d %d",
                &match_count, buf, buf, buf, buf, buf, buf, buf, strand,
                readName, &read_length, &read_from, &read_to, contigName, &contig_length, &contig_from, &contig_to
              );

        if (!is_multi && current_read_name == readName)
            continue;

        if (abs(read_from - read_to) < 0.8 * read_length)
            continue;

        if (current_read_name != readName)
        {
            ++total_aligned_reads;
            total_aligned_reads_length += abs(read_from - read_to);
        }
        //cout << total_aligned_reads << endl;
        //

        while(current_read_name != readName)
        {
            reader.Read(current_read, current_read_name);
            sscanf(current_read_name.c_str(), "%s", buf);
            current_read_name = buf;
        }

        //read_from;
        read_to--;
        //contig_from;
        contig_to--;

        int index = dict[contigName];

        int step = 1;
//        if (contig_from > contig_to)
//            step = -1;
        if (strand[0] == '-')
            step = -1;

        bool is_prev_error = false;
        //cout << current_read << endl;
        for (int i = 0; i < current_read.Size(); ++i)
        {
//            if (i < read_from || i > read_to)
//                ++errors[i];
//            else
            {
                int offset = i - read_from;
                int contig_index = contig_from + step * offset;

                if (strand[0] == '-')
                    contig_index = contig_to + step * offset;
                
                //cout << contigs[index][contig_index] << " " << contig_index << " ";
                if (strand[0] == '+')
                {
                    if (contigs[index][contig_index] != current_read[i])
                    {
                        ++errors[i];
                        if (is_prev_error)
                            ++partial[i];
                        is_prev_error = true;
                    }
                    else
                        is_prev_error = false;
                }
                else
                {
                    if (contigs[index][contig_index] != reverse_table[(int)current_read[i]])
                    {
                        ++errors[i];
                        if (is_prev_error)
                            ++partial[i];
                        is_prev_error = true;
                    }
                    else
                        is_prev_error = false;
                }
            }
        }

        if (contig_from > contig_to)
            swap(contig_from, contig_to);

        for (int i = contig_from; i <= contig_to; ++i)
            matchs[index][i]++;
    }

    long long sum_matchs = 0;
    long long sum_lengths = 0;
    long long sum_solid_lengths = 0;
    long long solid_count = 0;
    for (unsigned i = 0; i < matchs.size(); ++i)
    {
        sum_lengths += matchs[i].size();

        long long sum = 0;
        for (unsigned j = 0; j < matchs[i].size(); ++j)
        {
            sum += (matchs[i][j] > 0);
        }

        sum_matchs += sum;

        aligned_bases.push_back(sum);
        if (sum > 0.5 * matchs[i].size())
        {
            //sum_solid_lengths += matchs[i].size();
            sum_solid_lengths += sum;
            ++solid_count;
        }

//        ++cover_count[sum/matchs[i].size()];
//
//        cover_length[sum/matchs[i].size()] += matchs[i].size();
    }
    //printf("coverage %.2f, %lld\n", 1.0 * sum_matchs / sum_lengths, sum_lengths);

    printf("aligned reference %lld, total length %lld\n", solid_count, sum_solid_lengths);
    printf("aligned reads %lld\n", total_aligned_reads);
    printf("aligned reads length %lld\n", total_aligned_reads_length);

    for (unsigned i = 0; i < aligned_bases.size(); ++i)
    {
        //if (aligned_bases[i] >= 0.5 * matchs[i].size())
        printf("%d, %d, %d\n", i, aligned_bases[i], (int)matchs[i].size());

        for (unsigned j = 0; j < matchs[i].size(); ++j)
            printf("%d ", matchs[i][j]);
        printf("\n");
    }

    return 0;
}

