#include <iostream>
#include <algorithm>
#include <cstdio>
#include <string>
#include <vector>
#include <map>
#include <cstring>

using namespace std;

const int MaxLine = 1000000;

int min_nonzero = 0;
int max_nonzero = 200;
char line[MaxLine];
const char * taxonLevel[] =
{
    "kingdom", "phylum", "class", "order", "family", "genus", "species", 
};
vector<double> rates[7];

void Split(const string &s, vector<string> &items, int delimit)
{
    char buf[10000];

    strcpy(buf, s.c_str());

    vector<char *> pointers;
    bool is_new = true;
    for (unsigned i = 0; i < s.size(); ++i)
    {
        if (buf[i] != delimit)
        {
            if (is_new)
            {
                is_new = false;
                pointers.push_back(buf + i);
            }
        }
        else
        {
            is_new = true;
            buf[i] = '\0';
        }
    }

    for (unsigned i = 0; i < pointers.size(); ++i)
    {
        items.push_back(pointers[i]);
    }
}

void PrintDist(vector<double> &v)
{
    int count[101] = {0};

    for (unsigned i = 0; i < v.size(); ++i)
    {
        ++count[int(v[i] + 0.5)];
    }

    for (int i = 0; i <= 100; ++i)
    {
        printf("%d%%, %d\n", i, count[i]);
    }
    printf("\n");
}

struct TaxonRecord
{
    bool is_aligned;
    map<string, string> id_map;

    void Parse(const string &s)
    {
        vector<string> items;
        Split(s, items, '\t');

        if (items[9] != "-")
            is_aligned = true;
        else
            is_aligned = false;

        //cout << items[7] << " " << items[8] << " " << items[9] << " " << items[10] << endl;
        vector<string> values;
        vector<string> levels;
        Split(items[9], values, '|');
        Split(items[10], levels, '|');

        for (unsigned i = 0; i < levels.size(); ++i)
        {
            if (levels[i] != "-")
                id_map[levels[i]] = values[i];
            //cout << levels[i] << " " << values[i] << endl;
        }
        //cout << endl;

    }
};

vector<TaxonRecord> records;

int main(int argc, char *argv[])
{
    records.reserve(5000000);

    FILE *falign = fopen(argv[1], "rb");

    while (fgets(line, MaxLine, falign) != NULL)
    {
        line[strlen(line)-1] = '\0';

        TaxonRecord record;
        record.Parse(line);
        records.push_back(record);

        if (records.size() % 10000 == 0)
            printf("read aligend %d records\n", (int)records.size());
    }

    FILE *fcluster = fopen(argv[2], "rb");
    int num_clusters;
    fscanf(fcluster, "%d", &num_clusters);

    int min_size = atoi(argv[3]);

    for (int i = 0; i < num_clusters; ++i)
    {
        int id;
        int size;
        fscanf(fcluster, "%d %d", &id, &size);
        
        vector<int> cluster(size);
        for (int i = 0; i < size; ++i)
            fscanf(fcluster, "%d", &cluster[i]);

        if (size < min_size)
            continue;

        int non_zero = 0;
        if (non_zero < min_nonzero || non_zero > max_nonzero)
            continue;

        int num_aligned = 0;
        for (int i = 0; i < size; ++i)
        {
            if (records[cluster[i]].is_aligned)
                ++num_aligned;
        }

        printf("%d, %d, %d", id, size, num_aligned);

        for (int i = 0; i < 7; ++i)
        {
            map<string, int> taxon_count;
            for (int j = 0; j < size; ++j)
            {
                string value = records[cluster[j]].id_map[taxonLevel[i]];
                if (value != "" && value != "-")
                    taxon_count[value]++;
            }

            if (taxon_count.size() > 0)
            {
                string best = taxon_count.begin()->first;
                map<string, int>::iterator iter = taxon_count.begin();
                while (iter != taxon_count.end())
                {
                    if (iter->second > taxon_count[best])
                        best = iter->first;
                    ++iter;
                }

                printf(", %s, %.2f%%, %.2f%%", best.c_str(), 
                        taxon_count[best] * 100.0 / size,
                        taxon_count[best] * 100.0 / num_aligned);

                rates[i].push_back(taxon_count[best] * 100.0 / num_aligned);
            }
            else
            {
                printf(", -, 0, 0");
                rates[i].push_back(0);
            }
        }

        printf("\n");
    }

    for (int i = 0; i < 7; ++i)
    {
        cout << rates[i].size() << endl;
        PrintDist(rates[i]);
    }

    return 0;
}

