#include "config.h"

#include "globals.h"
#include "Log.h"
#include "Sequence.h"
#include "SimpleAssembler.h"
#include "IterativeAssembler.h"
#include "ScaffoldAssembler.h"
#include "GappedScaffoldAssembler.h"
#include "IDBAOption.h"
#include "Reader.h"
#include "Read.h"
#include "Writer.h"
#include "Utils.h"

#include <cstdio>
#include <algorithm>
#include <cstring>
#include <vector>
#include <cmath>
#include <string>


using namespace std;

int num[MaxLine];
int num2[MaxLine];
int num3[MaxLine];

int main(int argc, char *argv[])
{
    AddOption("kmer", "k", kmerLength, "");
    ProcessOptions(argc, argv);

    if (argc < 3)
    {
        fprintf(stderr, "kmerDist read-file ref-file\n");
        exit(1);
    }

    vector<Read> reads;
    FastAReader read_reader(argv[1]);
    FastAWriter read_writer(argv[2]);

    ((Reader&)read_reader).Read(reads);

    HashGraph *hash_graph = new HashGraph();

    {
#pragma omp parallel for
        for (int i = 0; i < (int64)reads.size(); ++i)
        {
            if (!reads[i].IsActive() || reads[i].Size() < kmerLength)
                continue;

            Sequence seq;
            seq = reads[i];
            hash_graph->InsertSequence(seq);
        }

        cout << "table size " << hash_graph->NumNodes() << " " << hash_graph->Capacity() << endl;
    }

    hash_graph->Refresh(2);

    cout << kmerLength << " " << hash_graph->NumNodes() << endl;
    hash_graph->RemoveDeadEnd(100);
    hash_graph->RemoveLowCoverageContigs(sqrt(hash_graph->AverageCoverage()));

    int index = 0;
    int count = 0;
    for (int i = 0; i < (int64)reads.size(); ++i)
    {
        if (!reads[i].IsActive() || reads[i].Size() < kmerLength)
            continue;

        Sequence seq;
        seq = reads[i];
        Kmer kmer;
        for (int i = 0; i < kmerLength; ++i)
            kmer.AddRight(seq[i]);

        int flag = false;
        int error = 0;
        while (true)
        {
            flag = hash_graph->Correct(seq, kmer, kmerLength, error);
            if (flag || error == 6)
                break;
            else
                ++error;
        }

        if (flag)
            ++count;

        seq.Decode();
        read_writer.WriteFormat(seq, "read%d", index++);
        fflush(NULL);
    }

    cout << count << " " << reads.size() << endl;

    return 0;
}

