#include "config.h"

#include "globals.h"
#include "Log.h"
#include "Sequence.h"
#include "SimpleAssembler.h"
#include "IterativeAssembler.h"
#include "ScaffoldAssembler.h"
#include "GappedScaffoldAssembler.h"
#include "IDBAOption.h"
#include "Reader.h"
#include "Read.h"
#include "Writer.h"
#include "Utils.h"

#include <cstdio>
#include <algorithm>
#include <cstring>
#include <vector>
#include <cmath>
#include <string>


using namespace std;

int num[MaxLine];
int num2[MaxLine];
int num3[MaxLine];
int min_count = 2;

int main(int argc, char *argv[])
{
    AddOption("kmer", "k", kmerLength, "");
    AddOption("minCount", "m", min_count, "");
    ProcessOptions(argc, argv);

    if (argc < 3)
    {
        fprintf(stderr, "kmerDist read-file ref-file\n");
        exit(1);
    }

    vector<Read> reads;
    FastAReader read_reader(argv[1]);

    ((Reader&)read_reader).Read(reads);

    HashGraph *hash_graph = new HashGraph();

    {
#pragma omp parallel for
        for (int i = 0; i < (int64)reads.size(); ++i)
        {
            if (!reads[i].IsActive() || reads[i].Size() < kmerLength)
                continue;

            Sequence seq;
            seq = reads[i];
            hash_graph->InsertSequence(seq);
        }

        cout << "table size " << hash_graph->NumNodes() << " " << hash_graph->Capacity() << endl;
    }

    for (HashGraph::Iterator iter = hash_graph->Begin(); 
            iter != hash_graph->End();
            ++iter)
    {
        ++num3[(*iter)->Count()];
    }

    hash_graph->Refresh(min_count);

    cout << kmerLength << " " << hash_graph->NumNodes() << endl;

//    {
//#pragma omp parallel for
//        for (int i = 0; i < (int64)reads.size(); ++i)
//        {
//            if (!reads[i].IsActive() || reads[i].Size() < kmerLength)
//                continue;
//
//            Sequence seq;
//            seq = reads[i];
//            hash_graph->AddInternalKmers(seq);
//        }
//
//        cout << "table size " << hash_graph->NumNodes() << " " << hash_graph->Capacity() << endl;
//    }

    hash_graph->RemoveDeadEnd(100);
    hash_graph->RemoveLowCoverageContigs(sqrt(hash_graph->AverageCoverage()));

    for (HashGraph::Iterator iter = hash_graph->Begin(); 
            iter != hash_graph->End();
            ++iter)
    {
        ++num[(*iter)->Count()];
    }

    hash_graph->SetDeadFlag();
    FastAReader ref_reader(argv[2]);
    Sequence ref;
    string comment;
    ref_reader.Read(ref, comment);
    ref.Encode();
    Kmer kmer;
    for (int i = 0; i < kmerLength-1; ++i)
        kmer.AddRight(ref[i]);

    for (int i = kmerLength-1; i < ref.Size(); ++i)
    {
        kmer.AddRight(ref[i]);
        KmerNode *node = hash_graph->GetNode(kmer);
        if (node != NULL)
            node->ResetDeadFlag();
        else
            ++num2[0];
    }

    hash_graph->Refresh(0);
    for (HashGraph::Iterator iter = hash_graph->Begin(); 
            iter != hash_graph->End();
            ++iter)
    {
        ++num2[(*iter)->Count()];
    }

    for (int i = 0; i < 1000; ++i)
        cout << i << " " << num3[i] << " " << num[i] << " " << num2[i] << endl;

    return 0;
}

