#include "globals.h"
#include "Sequence.h"
#include "Utils.h"
#include "BitOperation.h"
#include "Reader.h"
#include "KmerVector.h"

#include <cstdio>
#include <algorithm>
#include <cstring>
#include <iostream>
#include <cmath>
#include <ctime>
#include <vector>
#include <map>


using namespace std;

const string Version = "3.0";

int MaxReads = -1;
const int Times = 40;
const double minDist = 1e100;
const double p = 3.0;
struct Edge
{
    int from, to;
    double distance;

    bool operator <(const Edge &e) const
    {
        return distance < e.distance;
    }
};


bool is_merge = false;
double threshold  = 300;
int aux[MaxLine];
double dist[MaxLine];
int matrix[100][100];


bool isFasta = false;
bool isClassified = true;
bool isValidate = false;
int mapping[MaxLine];
int flags[MaxLine];
double sd[MaxLine];
char line[MaxLine];
int target_classes = 2;
int classes = 2;
KmerVector* kmerVectors;//[MaxReads];
int* type;//[MaxReads];
int* group;//[MaxReads];
KmerVector* center;//[MaxReads];
double* distSum;//[MaxReads];
double* distSumSquare;//[MaxReads];
double* distMean;//[MaxReads];
double* distSD;//[MaxReads];
double* distNum;//[MaxReads];
double* tmp;//[MaxReads];
string* comments;//[MaxReads];
Sequence* reads;//[MaxReads];
int* best;//[MaxReads];
int* isOutlier;//[MaxReads];
int total;

string taxon;
int* refTaxonID;//[MaxReads];
int* taxonID;//[MaxReads] = {0};
int* flagsType;//[MaxReads] = {0};
int* flagsGroup;//[MaxReads] = {0};

int assign[100];
int predictID[100];


char buf[MaxLine];
char name[MaxLine];
char attribute[MaxLine];
map<string, vector<int> > taxonDict;
///////////////////////////////////
vector<Edge> edges;
vector<double> sqredges;
//////////////////////////////
bool tofile = false;
double* intradistance;
double* avesumsqu;
//double* variance;
bool hasabovesh=false;
double cret_shre=0.9;
int *clusternum = NULL;
int *clusize = NULL;
int oriclasses=0;
FILE* myfp3=NULL;
double kmeanacc=0,kmeanta=0;

int MapSize=14;
double greymap[]={4.155286,
4.429123,
5.000324,
5.566155,
6.193574,
6.765069,
7.460029,
7.707445,
8.065385,
8.614547,
9.185615,
9.703879,
10.31519,
18.514029};
///////////////////////////////
string taxonNames[] =
{
    "phylum", "class", "order", "family", "genus", "species",
};

vector<string> refList;
vector<string> allRefs;
vector<KmerVector> allKmerVectors;
map<string, int> refRank;
int cluster_size = 1000;
int parent[1000000] = {0};

int Root(int x)
{
    if (parent[x] != x)
        parent[x] = Root(parent[x]);
    return parent[x];
}

void Merge(int x, int y)
{
    if (Root(x) != Root(y))
        parent[parent[x]] = parent[y];
}
int greylevel(double value)
{
	int greylev=0;
	while(value*1000>greymap[greylev] && greylev<MapSize)
		++greylev;
	return greylev;
}
double Distance(KmerVector *kv1, KmerVector *kv2)
{
    double sum = 0;
    for (int i = 0; i < kv1->size; ++i)
    /////////test for barcode 
    sum += abs(kv1->ranks[i] - kv2->ranks[i]);
        
 // sum += pow((greylevel(kv1->values[i]/s1) - greylevel(kv2->values[i]/s2)),2);
    /////////end test
    return sum;
}

double DistanceAverage(vector<int> &kv1, vector<int> &kv2)
{
    vector<double> sum(kv1.size());
#pragma omp parallel for
    for (int i = 0; i < (int)kv1.size(); ++i)
    {
        sum[i] = 0;
        for (int j = 0; j < (int)kv2.size(); ++j)
        {
            double d = Distance(&kmerVectors[kv1[i]], &kmerVectors[kv2[j]]);
            sum[i] += d;
        }
    }

    double total = 0;
    for (unsigned i = 0; i < sum.size(); ++i)
        total += sum[i];

    int64 count = kv1.size() * kv2.size();
    return total / count;
}
double IntraDistance(vector<int> &kv1)
{
    vector<double> sum(kv1.size());
#pragma omp parallel for
    for (int i = 0; i < (int)kv1.size(); ++i)
    {
        sum[i] = 0;
        for (int j = i+1; j < (int)kv1.size(); ++j)
        {
            double d = Distance(&kmerVectors[kv1[i]], &kmerVectors[kv1[j]]);
            sum[i] += d;
        }
    }

    double total = 0;
    for (unsigned i = 0; i < sum.size(); ++i)
        total += sum[i];

    int64 count = kv1.size() * (kv1.size()-1)/2;
    return total / count;
}
///////////////////////////////////////////////////////////////////
double AveSquareDis(vector<int> &kv1, vector<int> &kv2)
{
    vector<double> sum(kv1.size());
#pragma omp parallel for
    for (int i = 0; i < (int)kv1.size(); ++i)
    {
        sum[i] = 0;
        for (int j = 0; j < (int)kv2.size(); ++j)
        {
            double d = Distance(&kmerVectors[kv1[i]], &kmerVectors[kv2[j]]);
            sum[i] += d*d;
        }
    }

    double total = 0;
    for (unsigned i = 0; i < sum.size(); ++i)
        total += sum[i];

    int64 count = kv1.size() * kv2.size();
    return total / count;
}
double IntraAveSquareDis(vector<int> &kv1)
{
    vector<double> sum(kv1.size());
#pragma omp parallel for
    for (int i = 0; i < (int)kv1.size(); ++i)
    {
        sum[i] = 0;
        for (int j = i+1; j < (int)kv1.size(); ++j)
        {
            double d = Distance(&kmerVectors[kv1[i]], &kmerVectors[kv1[j]]);
            sum[i] += d*d;
        }
    }

    double total = 0;
    for (unsigned i = 0; i < sum.size(); ++i)
        total += sum[i];

    int64 count = kv1.size() * (kv1.size()-1)/2;
    return total / count;
}
///////////////////////////////////////////////////////////////////
void Vote()
{
    for (int i = 0; i < classes; ++i)
    {
        for (int j = 0; j < classes; ++j)
            matrix[i][j] = 0;
    }
    for (int i = 0; i < total; ++i)
    {
        if (!flagsType[type[i]] && !isOutlier[i] && !flagsGroup[group[i]])
        {
            ++matrix[type[i]][group[i]];
        }
    }
}

void GetLabel(int &t, int &l)
{
    t = -1;
    l = -1;
    for (int i = 0; i < classes; ++i)
    {
        for (int j = 0; j < classes; ++j)
        {
            if (!flagsType[i] && !flagsGroup[j] && (t == -1 || matrix[i][j] > matrix[t][l]))
            {
                t = i;
                l = j;
            }
        }
    }

}

double DistributePoints()
{
    double power = 0;
    for (int i = 0; i < classes; ++i)
    {
        distSum[i] = 0;
        distSumSquare[i] = 0;
        distNum[i] = 0;
    }

#pragma omp parallel for
    for (int i = 0; i < total; ++i)
    {
        if (isOutlier[i])
            continue;

        double minimum = 1e100;
        int t = -1;
        for (int j = 0; j < classes; ++j)
        {
            double d = Distance(&kmerVectors[i], &center[j]);
            if (d < minimum)
            {
                minimum = d;
                t = j;
            }
        }

        if (minimum < minDist)
        {
            type[i] = t;
#pragma omp atomic
            power += minimum;
#pragma omp atomic
            distSum[t] += minimum;
#pragma omp atomic
            distSumSquare[t] += minimum * minimum;
#pragma omp atomic
            distNum[t]++;
        }
        else
            type[i] = -1;
    }

    for (int i = 0; i < classes; ++i)
    {
        distMean[i] = distSum[i] / distNum[i];
        distSD[i] = sqrt((distSumSquare[i] - distSum[i]*distMean[i]) / (distNum[i] - 1));
    }

    int count = 0;
    for (int i = 0; i < total; ++i)
    {
        if (isOutlier[i] != true)
            ++count;
    }

    return power / count;
}

bool CalCenter()
{
    bool isUpdate = false;
    for (int j = 0; j < classes; ++j)
    {
        KmerVector sum;
        int count = 0;
        for (int i = 0; i < total; ++i)
        {
            if (!isOutlier[i] && type[i] == j)
            {
                for (int l = 0; l < sum.size; ++l)
                    sum.values[l] += kmerVectors[i].values[l];
                ++count;
            }
        }

        for (int l = 0; l < sum.size; ++l)
            sum.values[l] /= count;

        for (int l = 0; l < center[j].size; ++l)
        {
            if (fabs(center[j].values[l] - sum.values[l]) > 1e-6)
                isUpdate = true;
            center[j].values[l] = sum.values[l];
        }

        center[j].ComputeRank();
    }

    return isUpdate;
}

double Kmeans()
{
    // Randomly select the cluster center.
    for (int i = 0; i < total; ++i)
    {
        aux[i] = i;
        isOutlier[i] = false;
        type[i] = -1;
    }

    for (int i = 0; i < classes; ++i)
    {
        int j = i + rand()%(total-i);
        swap(aux[i], aux[j]);
        center[i] = kmerVectors[aux[i]];
    }

    double power = 0;
    for (int k = 0; k < Times; ++k)
    {
        power = DistributePoints();

        if (CalCenter() == false)
            break;
    }

    power = DistributePoints();

    int count = 0;
    for (int i = 0; i < total; ++i)
    {
        if (isOutlier[i])
            continue;

        double d = Distance(&kmerVectors[i], &center[type[i]]);
        if (d > distMean[type[i]] + 2 * distSD[type[i]])
        {
            isOutlier[i] = true;
            ++count;
        }
    }
    power = 0;
    for (int k = 0; k < Times; ++k)
    {
        power = DistributePoints();

        if (CalCenter() == false)
            break;
    }
    return power;
}

void Normalize()
{
    int size = kmerVectors[0].size;
    for (int i = 0; i < size; ++i)
    {
        double sum = 0;
        double sumSquare = 0;
        for (int j = 0; j < total; ++j)
        {
            sum += kmerVectors[j].values[i];
            sumSquare += kmerVectors[j].values[i] * kmerVectors[j].values[i];
        }
        double mean = sum / total;
        double sd = sqrt((sumSquare - sum*mean) / (total - 1));
        for (int j = 0; j < total; ++j)
        {
            kmerVectors[j].values[i] -= mean;
            kmerVectors[j].values[i] /= sd;
        }
    }
}
/////////////////////////
int calfromto(int from, int to,int classes)
{
    if(from<to)
   return  from*classes-from*(from+3)/2+to-1;
   else return to*classes-to*(to+3)/2+from-1;
}
/////////////////////////
void MergeClusters()
{
    vector<int> clusters[classes];
    for (int i = 0; i < total; ++i)
    {
        if (!isOutlier[i])
            clusters[type[i]].push_back(i);
    }

    for (int i = 0; i < classes; ++i)
    {
        for (int j = i+1; j < classes; ++j)
        {
            Edge e;
            e.from = i;
            e.to = j;
            e.distance = DistanceAverage(clusters[i], clusters[j]);
            edges.push_back(e);
            sqredges.push_back(AveSquareDis(clusters[i], clusters[j]));
        }
    }
    


 //   cout << "calculate distance" << endl;

//    sort(edges.begin(), edges.end());

 //   cout << "sort distance" << endl;

    for (int i = 0; i < classes; ++i)
        parent[i] = i;

    int current_classes = classes;
    oriclasses = classes;

    clusternum = new int[classes];
    clusize = new int[classes];
     for(int i= 0;i<classes;i++)
     {
        clusternum[i]=i;
        clusize[i]=clusters[i].size();

     }
     
    for (int i = 0; i < classes; ++i)
    {
        intradistance[i] = IntraDistance(clusters[i]);
        avesumsqu[i] = IntraAveSquareDis(clusters[i]);
//        variance[i] = avesumsqu[i]-intradistance[i]*intradistance[i];
    }
    
     
//,cret_shre
    ////////////////////////////////////
    for (unsigned i = 0; i < edges.size(); ++i)
    {
        if (current_classes == 1 )
            break;
        hasabovesh = false;    
        double disttmp = 99999;
        int tobemerge = -1;
        int from,to;
        for (unsigned j = 0; j < edges.size(); ++j)
        {
            from = edges[j].from;
            to = edges[j].to;
            if(edges[j].distance<disttmp && Root(edges[j].from) != Root(edges[j].to) && (intradistance[from]+intradistance[to])/edges[j].distance>cret_shre*2)
            {
                disttmp = edges[j].distance;
                tobemerge = j;
                hasabovesh = true;
            }
        }
        if(hasabovesh==false)break;
        from = edges[tobemerge].from;
        to = edges[tobemerge].to;
        Merge(from, to);
        --current_classes;

        ///////////////////////////////////
        double totaldis = edges[tobemerge].distance*clusize[from]*clusize[to];
        totaldis += intradistance[from]*clusize[from]*(clusize[from]-1)/2;
        totaldis += intradistance[to]*clusize[to]*(clusize[to]-1)/2;
        totaldis /= ((double)(clusize[from]+clusize[to])*(double)(clusize[from]+clusize[to]-1)/2);
        for (int j = 0; j < classes; ++j)
        {   
            if(clusternum[j]==clusternum[from] || clusternum[j]==clusternum[to])
            {
                intradistance[j] = totaldis ;
            }
        }
        ////////////////////////////// for square
        double totalsqrdis = sqredges[tobemerge]*clusize[from]*clusize[to];
        totalsqrdis += avesumsqu[from]*clusize[from]*(clusize[from]-1)/2;
        totalsqrdis += avesumsqu[to]*clusize[to]*(clusize[to]-1)/2;
        totalsqrdis /= ((double)(clusize[from]+clusize[to])*(double)(clusize[from]+clusize[to]-1)/2);
        for (int j = 0; j < classes; ++j)
        {   
            if(clusternum[j]==clusternum[from] || clusternum[j]==clusternum[to])
            {
                avesumsqu[j] = totalsqrdis ;
            }
        }
        ///////////////////////////////
        
        for (unsigned j = 0; j < edges.size(); ++j)
        {
            double tmpd1,tmpd2;
            int size1,size2;
            if(j==tobemerge)continue;
            if(edges[j].to == from)
            {
                tmpd1 = edges[j].distance;
                tmpd2 = edges[calfromto(edges[j].from,to,classes)].distance;
                size1 = clusize[from];
                size2 = clusize[to];
                edges[j].distance = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].from == from)
            {
                tmpd1 = edges[j].distance;
                tmpd2 = edges[calfromto(edges[j].to,to,classes)].distance;
                size1 = clusize[from];
                size2 = clusize[to];
                edges[j].distance = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].to == to)
            {
                tmpd1 = edges[calfromto(edges[j].from,from,classes)].distance;
                tmpd2 = edges[j].distance;
                size1 = clusize[from];
                size2 = clusize[to];
                edges[j].distance = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].from == to)
            {
                tmpd1 = edges[calfromto(from,edges[j].to,classes)].distance;
                tmpd2 = edges[j].distance;
                size1 = clusize[from];
                size2 = clusize[to];
                edges[j].distance = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
        }
        /////////////////////////////////////////////////////////////for square
        for (unsigned j = 0; j < edges.size(); ++j)
        {
            double tmpd1,tmpd2;
            int size1,size2;
            if(j==tobemerge)continue;
            if(edges[j].to == from)
            {
                tmpd1 = sqredges[j];
                tmpd2 = sqredges[calfromto(edges[j].from,to,classes)];
                size1 = clusize[from];
                size2 = clusize[to];
                sqredges[j] = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].from == from)
            {
                tmpd1 = sqredges[j];
                tmpd2 = sqredges[calfromto(edges[j].to,to,classes)];
                size1 = clusize[from];
                size2 = clusize[to];
                sqredges[j] = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].to == to)
            {
                tmpd1 = sqredges[calfromto(edges[j].from,from,classes)];
                tmpd2 = sqredges[j];
                size1 = clusize[from];
                size2 = clusize[to];
                sqredges[j] = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
            else if(edges[j].from == to)
            {
                tmpd1 = sqredges[calfromto(from,edges[j].to,classes)];
                tmpd2 = sqredges[j];
                size1 = clusize[from];
                size2 = clusize[to];
                sqredges[j] = (tmpd1*size1+tmpd2*size2)/(size1+size2);
            }
        }
        int temp = clusize[from]+clusize[to];
        for (int j = 0; j < classes; ++j)
        {   
            if(clusternum[j]==clusternum[from] || clusternum[j]==clusternum[to])
                clusize[j] = temp;
        }

        /////////////////////////////////////
        int tmp = clusternum[to];
        for(int j= 0;j<classes;j++)
        {
            if(clusternum[j]==tmp)
                clusternum[j]=clusternum[from];
        }
    }
    ///////////////////////////////////

    ///////////////////////////////////////////

    cout << "merge by distance" << endl;

    map<int, int> id;
    int index = 0;
    for (int i = 0; i < classes; ++i)
    {
        if (id.find(Root(i)) == id.end())
            id[Root(i)] = index++;
    }

    for (int i = 0; i < total; ++i)
    {
        if (!isOutlier[i])
            type[i] = id[Root(type[i])];
    }
    classes = id.size();

    cout << "relabel" << endl;
}

void Usage()
{
    printf("metaCluster %s\n", Version.c_str());
    printf("Usage: metacluster input_file [--threshold t]\n");
}

int main(int argc, char *argv[])
{
    srand((unsigned)time(NULL));

    AddParameter("threshold_t", &threshold, FLOAT);
    AddParameter("size", &cluster_size, INTEGER);
    AddParameter("threshold", &cret_shre,FLOAT);
    ProcessParameters(argc, argv);
    cout<<"cret_shre:"<<cret_shre<<endl;
    for(int i=0;i<argc;++i)
        cout<<"i:"<<argv[i]<<endl;
    if (argc < 2)
    {
        Usage();
        exit(1);
    }

    // Get the reads and compute the kmer vectors
    Sequence seq;
    unsigned index = 0;
    FastAReader reader(argv[1]);

    int num_suequence = reader.NumReads();
    //////////////////////////////////////////////////////////
	MaxReads = num_suequence;
	cout<<"MaxReads:"<<MaxReads<<endl;
	kmerVectors = new KmerVector[num_suequence];//[MaxReads];
	type = new int[num_suequence];//[MaxReads];
	group = new int[num_suequence];//[MaxReads];
	center = new KmerVector[num_suequence];//[MaxReads];
	distSum = new double[num_suequence];//[MaxReads];
	distSumSquare = new double[num_suequence];//[MaxReads];
	distMean = new double[num_suequence];//[MaxReads];
	distSD = new double[num_suequence];//[MaxReads];
	distNum = new double[num_suequence];//[MaxReads];
	tmp = new double[num_suequence];//[MaxReads];
	comments = new string[num_suequence];//[MaxReads];
	reads = new Sequence[num_suequence];//[MaxReads];
	best = new int[num_suequence];//[MaxReads];
	isOutlier = new int[num_suequence];//[MaxReads];
	refTaxonID = new int[num_suequence];//[MaxReads];
	taxonID = new int[num_suequence];//[MaxReads] = {0};
	flagsType = new int[num_suequence];//[MaxReads] = {0};
	flagsGroup = new int[num_suequence];//[MaxReads] = {0};
	for(int i=0;i<num_suequence;++i)
	{
		taxonID[i]=0;
		flagsType[i]=0;
		flagsGroup[i]=0;
	}
    
    //////////////////////////////////////////////////////////
    classes = num_suequence / cluster_size + (num_suequence % cluster_size > 0);
    cout << classes << endl;

    string comment;
    while (reader.Read(seq, comment))
    {
            if (seq.IsChar())
            {
                comments[index] = comment;
                reads[index] = seq;
                seq.Encode();
                kmerVectors[index].Compute(&seq);

                ++index;

                strcpy(line, comment.c_str());
                if (isValidate)
                {
                    for (int i = 0; line[i]; ++i)
                    {
                        if (line[i] == '_')
                        {
                            group[index] = atoi(line + i+1);
                            break;
                        }
                    }
                }
            }
    }

    total = index;

    double optimal = 1e100;
    for (int round = 0; round < 10; ++round)
    {
        double power = Kmeans();

        if (power < optimal)
        {
            optimal = power;
            copy(type, type + total, best);
        }
    }

    copy(best, best + total, type);
    fill_n(flags, MaxLine, 1);

    /////////////////////////////////////////////

    intradistance = new double[classes];
    avesumsqu = new double[classes];
    /////////////////////////////////////////////
    MergeClusters();


    int correct = 0;
    fill_n(flagsType, classes, 0);
    fill_n(flagsGroup, classes, 0);

        for (int t = 0; t < classes; ++t)
        {
            int count = 0;
            for (int i = 0; i < total; ++i)
            {
                if (!isOutlier[i] && type[i] == t)
                    ++count;
            }

            printf("group %03d, %d fragments\n", t, count);
        }

    if (isValidate)
    {
        int count = 0;
        int classCount = 0;
        for (int i = 0; i < total; ++i)
        {
            if (isOutlier[i])
                ++count;
            else if (taxonID[i] == refTaxonID[group[i]])
                ++classCount;
        }

        printf("clustering accuracy: %d/%d %.2f%%\n", correct, total - count, correct*100.0/(total - count));
        printf("classification accuracy: %.2f%%\n", classCount*100.0/(total - count));
    }
//////////////////////////////////////////10.1
    cout<<classes<<"\t"<<oriclasses<<endl;
    int leftclusters=0,leftfinal=0;
    //////////////////////
    int* newclusternum = new int[oriclasses];
    for(int i=0;i<oriclasses;i++)
        newclusternum[i]=-1;
    int current_va=0;
    for(int i=0;i<oriclasses;i++)
    {
        bool has =false;
        for(int j=0;j<i;j++)
        {
            if(clusternum[j]==clusternum[i])
            {
                has = true;
                break;
            }
        }
        if(!has && clusize[i]>=threshold)
        {
            newclusternum[current_va]=clusternum[i];
            current_va ++;  
        }
    }
    cout<<current_va<<"\tleft clusters:";
    for(int i=0;i<oriclasses && newclusternum[i]>-1;i++)
        cout<<newclusternum[i]<<"\t";
    cout<<endl;
    
        /////////////
//    int countlen=0;
    double sumall=0;
    for(int i=0;i<oriclasses;i++)
    {
        bool hasprint=false;
        for(int j=0;j<i;j++)
        {
            if(clusternum[j]==clusternum[i])
            {
                hasprint=true;
                break;
            }
        }
        if(!hasprint)
        {
            bool is_outl=false;
            if(clusize[i]<threshold)is_outl=true;
            leftclusters++;
            for(int j=i;j<oriclasses;j++)
                if(clusternum[j]==clusternum[i])
                    cout<<j<<",";
            cout<<endl;
            if(!is_outl)
            {
                leftfinal++;

                sumall += clusize[i];
            }
        }
    }
/////////////////////////new outliers (shreshold = 300)

    int* che = new int[oriclasses];
    for(int i=0;i<oriclasses;i++)che[i]=0;
    for (int i = 0; i < total; ++i)
    {
        if (!isOutlier[i])
            che[type[i]]++;
    }
    for(int i=0;i<oriclasses;i++)cout<<che[i]<<"\t";
    for (int i = 0; i < total; ++i)
    {
        if (!isOutlier[i] && che[type[i]]<=300)
            isOutlier[i]=true;
    }
/////////////////////////////////////////output 
    FILE **foutput = new FILE*[leftfinal];
    for (int i = 0; i < leftfinal; ++i)
    {
        sprintf(buf, "%s-out%03d.fa", argv[1], i);
        foutput[i] = fopen(buf, "wt");
        fflush(NULL);
    }
    for (int i = 0; i < total; ++i)
    {
        if (!isOutlier[i] && type[i]>=0 && type[i]<leftfinal)
        {
            FILE *fp = foutput[type[i]];
            fprintf(fp, ">%s\n", comments[i].c_str());
        }
    }
//////////////////////////////////////////////\Tom
//    delete[] foutput;
    delete [] clusternum; 
    delete []clusize;
    delete []intradistance;
    delete []newclusternum;
    
    return 0;
}

