#include "globals.h"
#include "Log.h"
#include "Sequence.h"
#include "Utils.h"
#include "Reader.h"
#include "Writer.h"

#include <cctype>
#include <cstdio>
#include <algorithm>
#include <cstring>
#include <iostream>
#include <vector>

using namespace std;


bool mate = false;
int insertLength = 250;
char line[MaxLine];
uint64 depth = 100;
double errorRate = 0.01;
int readLength = 35;
uint64 readNumber = 0;
int errorNums[1000] = {0};
bool separate = false;
vector<Sequence> refs;
vector<int64> numbers;
bool is_uniform = true;

void Simulate(Sequence &ref, uint64 total_number, FastAWriter &writer)
{
    if (ref.Size() < readLength)
        return;

    if (mate && ref.Size() < insertLength)
        return;

    if (!mate)
    {
        Sequence seq;
        for (unsigned i = 0; i < total_number; ++i)
        {
            unsigned offset = 0;
            while (true)
            {
                offset = rand() % (ref.Size() - readLength + 1);
                ref.GetSubSequence(seq, offset, readLength);
                if (seq.IsChar())
                    break;
            }

            seq.Encode();

            int error = 0;
            for (int j = 0; j < readLength; ++j)
            {
                if (rand()*1.0 < errorRate*RAND_MAX)
                {
                    ++error;
                    int c = seq[j];
                    while (c == seq[j])
                    {
                        c = rand()/93 % 4;
                    }
                    seq[j] = c;
                }
            }
            seq.Decode();

            ++errorNums[error];

            if (rand() < RAND_MAX/2)
                seq.ReverseComplement();

            writer.WriteFormat(seq, "read%d", i);
        }
    }
    else
    {
        Sequence seq1;
        Sequence seq2;
        for (unsigned i = 0; i < total_number; i += 2)
        {
            unsigned offset = 0;
            while (true)
            {
                offset = rand() % (ref.Size() + 1 - insertLength);
                ref.GetSubSequence(seq1, offset, readLength);
                ref.GetSubSequence(seq2, offset + insertLength - readLength, readLength);
                if (seq1.IsChar() && seq2.IsChar())
                    break;
            }

            seq1.Encode();
            seq2.Encode();
            int error1 = 0;
            int error2 = 0;
            for (int j = 0; j < readLength; ++j)
            {
                if (rand()*1.0 < errorRate*RAND_MAX)
                {
                    ++error1;
                    int c = seq1[j];
                    while (c == seq1[j])
                    {
                        c = rand()/93 % 4;
                    }
                    seq1[j] = c;
                }

                if (rand()*1.0 < errorRate*RAND_MAX)
                {
                    ++error2;
                    int c = seq2[j];
                    while (c == seq2[j])
                    {
                        c = rand()/93 % 4;
                    }
                    seq2[j] = c;
                }
            }

            ++errorNums[error1];
            ++errorNums[error2];

            seq1.Decode();
            seq2.Decode();
            seq2.ReverseComplement();

            if (i%2 == 0)
            {
                writer.WriteFormat(seq2, "read%d/1", i);
                writer.WriteFormat(seq1, "read%d/2", i);
            }
            else
            {
                writer.WriteFormat(seq1, "read%d/1", i);
                writer.WriteFormat(seq2, "read%d/2", i);
            }
        }
    }
}

int main(int argc, char *argv[])
{
    AddParameter("depth", &depth, INTEGER);
    AddParameter("errorRate", &errorRate, FLOAT);
    AddParameter("readLength", &readLength, INTEGER);
//     AddParameter("readNumber", &readNumber, INTEGER);
    AddParameter("mate", &mate, SIMPLE);
    AddParameter("insertLength", &insertLength, INTEGER);
    AddParameter("separate", &separate, SIMPLE);

    ProcessParameters(argc, argv);

    if (argc < 3
        || strcmp(argv[1], "--help") == 0
        || strcmp(argv[1], "-h") == 0)
    {
        fprintf(stderr, "usage: simRead ref-file read-file\n");
        fprintf(stderr, "       [--depth d] [--mate]\n");
        fprintf(stderr, "       [--errorRate e] [--readLength l]\n");
        exit(1);
    }


    FastAReader reader(argv[1]);
    FastAWriter writer(argv[2]);

    Sequence ref;
    string comment;
    Sequence genome;
    while (reader.Read(ref, comment))
    {
//        if (genome.Size() != 0)
//            genome += 'N';
//        genome += ref;
        for (int i = 0; i < ref.Size(); ++i)
        {
            ref[i] = toupper(ref[i]);
        }
        refs.push_back(ref);
    }
//    ref = genome;

//    for (unsigned i = 0; i < ref.Size(); ++i)
//    {
//        ref[i] = toupper(ref[i]);
//    }

    int64 total_length = 0;
    for (unsigned i = 0; i < refs.size(); ++i)
    {
        total_length += refs[i].Size();
    }

    if (readNumber == 0)
        readNumber = (depth * total_length) / readLength;
    else
        depth = int(readLength*readNumber/total_length);

    readNumber += readNumber % 2;
    LogMessage("depth %d\treadLength %d\terrorRate %.3f\treadNumber %d\n",
        depth, readLength, errorRate, readNumber);
    LogMessage("insertLength %d\n", insertLength);
    LogMessage("mate %d\n", mate);

    numbers.resize(refs.size());

    int64 sum = 0;
    for (unsigned i = 0; i < refs.size(); ++i)
    {
        //numbers[i] = rand() % 1000;
        numbers[i] = 1;
        sum += numbers[i] * refs[i].Size();
    }

    for (unsigned i = 0; i < refs.size(); ++i)
    {
        numbers[i] = numbers[i] * refs[i].Size() * readNumber / sum;
    }

    for (unsigned i = 0; i < refs.size(); ++i)
    {
        Simulate(refs[i], numbers[i], writer);
    }

    return 0;
}

