
#include "globals.h"
#include "Log.h"
#include "Sequence.h"
#include "HashGraph.h"
#include "ContigGraph.h"
#include "HashAlign.h"
#include "MapGraph.h"
#include "SimpleAlgorithm.h"

//#include <boost/program_options.hpp>

#include <cstdio>
#include <algorithm>
#include <cstring>
#include <vector>
#include <cmath>
#include <string>

//namespace po = boost::program_options;

using namespace std;

SimpleAlgorithm umea;

void Usage()
{
    cout << "IDBA: Iterative De Bruijn graph short read Assembler" << endl;
    cout << "Version " << MajorVersion << "." << MinorVersion << endl;
    cout << endl;
    cout << "Usage: umea --read read-file [--output out] [options]\n" << endl;
}

int main(int argc, char *argv[])
{
    po::options_description desc("Allowed options");

    desc.add_options()
        ("help,h", "produce help message")
        ("read,r", po::value<string>(&umea.readfile), "read file")
        ("output,o", po::value<string>(&umea.prefix)->default_value("out"), "prefix of output")
        ("mink", po::value<int>(&umea.mink)->default_value(25), "minimum k value")
        ("minCount", po::value<int>(&umea.minCount)->default_value(2), "filtering threshold for each k-mer")
        ("cover", po::value<double>(&umea.cover)->default_value(0), "the cutting coverage for contigs")
        ("trim", po::value<int>(&umea.trim)->default_value(0), "discard n bases a the end of each read")
    ;

    try
    {
        po::variables_map vm;
        po::store(po::parse_command_line(argc, argv, desc), vm);
        po::notify(vm);

        umea.isScaffold = vm.count("scaffold");

        if (vm.count("help") || vm.count("read") == 0)
        {
            Usage();
            cout << desc << endl;
            return 1;
        }
    }
    catch (exception &e)
    {
        Usage();
        cout << desc << endl;
        return 1;
    }

    umea.Run();

    return 0;
}
